@@ -23,7 +23,6 @@
 #include <config.h>
 #include <string.h>
 #include "gtk-utils.h"
-#include <libgnome/gnome-help.h>
 
 #define LOAD_BUFFER_SIZE 65536
 
@@ -90,7 +89,7 @@ _gtk_message_dialog_new (GtkWindow        *parent,
 
 	if (message != NULL) {
 		char *escaped_message;
-		
+
 		escaped_message = g_markup_escape_text (message, -1);
 		if (secondary_message != NULL) {
 			char *escaped_secondary_message = g_markup_escape_text (secondary_message, -1);
@@ -98,17 +97,17 @@ _gtk_message_dialog_new (GtkWindow        *parent,
 						       escaped_message,
 						       escaped_secondary_message);
 			g_free (escaped_secondary_message);
-		} 
+		}
 		else
 			markup_text = g_strdup (escaped_message);
 		g_free (escaped_message);
 	}
-	else 
+	else
 		markup_text = g_markup_escape_text (secondary_message, -1);
-	 
+
 	gtk_label_set_markup (GTK_LABEL (label), markup_text);
 	g_free (markup_text);
-	
+
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
 
@@ -268,7 +267,7 @@ _gtk_request_dialog_run (GtkWindow        *parent,
 
 	button = create_button (GTK_STOCK_CANCEL, no_button_text);
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
-				      button, 
+				      button,
 				      GTK_RESPONSE_CANCEL);
 
 	button = create_button (GTK_STOCK_OK, yes_button_text);
@@ -343,14 +342,14 @@ _gtk_yesno_dialog_new (GtkWindow        *parent,
 
 	button = create_button (GTK_STOCK_CANCEL, no_button_text);
 	gtk_dialog_add_action_widget (GTK_DIALOG (d),
-				      button, 
+				      button,
 				      GTK_RESPONSE_CANCEL);
 
 	/**/
 
 	button = create_button (GTK_STOCK_OK, yes_button_text);
 	gtk_dialog_add_action_widget (GTK_DIALOG (d),
-				      button, 
+				      button,
 				      GTK_RESPONSE_YES);
 
 	/**/
@@ -361,16 +360,6 @@ _gtk_yesno_dialog_new (GtkWindow        *parent,
 }
 
 
-static void
-toggle_visibility (GtkWidget *widget)
-{
-	if (GTK_WIDGET_VISIBLE (widget))
-		gtk_widget_hide (widget);
-	else
-		gtk_widget_show (widget);
-}
-
-
 GtkWidget*
 _gtk_error_dialog_new (GtkWindow        *parent,
 		       GtkDialogFlags    flags,
@@ -386,7 +375,7 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 	GtkWidget     *vbox;
 	GtkWidget     *text_view;
 	GtkWidget     *scrolled = NULL;
-	GtkWidget     *button;
+	GtkWidget     *expander;
 	GtkTextBuffer *text_buf;
 	GtkTextIter    iter;
 	char          *stock_id;
@@ -398,7 +387,7 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 	stock_id = GTK_STOCK_DIALOG_ERROR;
 
 	dialog = gtk_dialog_new_with_buttons ("",
-					      parent, 
+					      parent,
 					      flags,
 					      GTK_STOCK_OK, GTK_RESPONSE_OK,
 					      NULL);
@@ -431,7 +420,7 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 		va_end (args);
 		escaped_secondary_message = g_markup_escape_text (secondary_message, -1);
 
-		markup_text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+		markup_text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n",
 					       escaped_message,
 					       escaped_secondary_message);
 
@@ -445,9 +434,9 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 	g_free (escaped_message);
 
 	if (view_output) {
-		/* Button */
+		/* Expander */
 
-		button = gtk_toggle_button_new_with_mnemonic (_("Command _Line Output"));
+		expander = gtk_expander_new_with_mnemonic (_("Command _Line Output"));
 
 		/* Add text */
 
@@ -487,40 +476,22 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 	vbox = gtk_vbox_new (FALSE, 6);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 
-	hbox = gtk_hbox_new (FALSE, 6);
-	gtk_box_pack_start (GTK_BOX (hbox), image,
-			    FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), label,
-			    FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox,
-			    TRUE, TRUE, 0);
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
 
 	if (view_output) {
-		hbox = gtk_hbox_new (FALSE, 6);
-		gtk_box_pack_start (GTK_BOX (hbox), button,
-			    FALSE, FALSE, 0);
-		gtk_box_pack_start (GTK_BOX (vbox), hbox,
-				    TRUE, TRUE, 0);
-
 		gtk_container_add (GTK_CONTAINER (scrolled), text_view);
-		gtk_box_pack_start (GTK_BOX (vbox), scrolled,
-				    FALSE, FALSE, 0);
-
-		g_signal_connect_swapped (G_OBJECT (button),
-					  "clicked", 
-					  G_CALLBACK (toggle_visibility), 
-					  scrolled);				    
+		gtk_container_add (GTK_CONTAINER (expander), scrolled);
+		gtk_box_pack_start (GTK_BOX (vbox), expander, TRUE, TRUE, 0);
 	}
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    vbox,
 			    FALSE, FALSE, 0);
-
 	gtk_widget_show_all (vbox);
 
-	if (scrolled != NULL)
-		gtk_widget_hide (scrolled);
-
 	return dialog;
 }
 
@@ -679,156 +650,69 @@ _gtk_label_get_filename_text (GtkLabel   *label)
 	return text;
 }
 
-/* This function from gnome-panel/panel-util.c
- * (C) 1997, 1998, 1999, 2000 The Free Software Foundation
- * Copyright 2000 Helix Code, Inc.
- * Copyright 2000,2001 Eazel, Inc.
- * Copyright 2001 George Lebl
- * Copyright 2002 Sun Microsystems Inc.
- *
- * Authors: George Lebl
- *          Jacob Berkman
- *          Mark McLoughlin
- *
- * Modified by Paolo Bacchilega for the Quick Lounge applet
- */
-static char*
-panel_find_icon (GtkIconTheme  *icon_theme,
-		 const char    *icon_name,
-		 gint           size)
-{
-	char        *retval  = NULL;
-	GtkIconInfo *icon_info = NULL;
-	char        *icon_no_extension;
-	char        *p;
 
-
-	if (icon_name == NULL || strcmp (icon_name, "") == 0)
-		return NULL;
-
-	if (g_path_is_absolute (icon_name)) {
-		if (g_file_test (icon_name, G_FILE_TEST_EXISTS)) {
-			return g_strdup (icon_name);
-		} else {
-			char *basename;
-
-			basename = g_path_get_basename (icon_name);
-			retval = panel_find_icon (icon_theme,
-						  basename,
-						  size);
-			g_free (basename);
-
-			return retval;
-		}
-	}
-
-	/* This is needed because some .desktop files have an icon name *and*
-	 * an extension as icon */
-	icon_no_extension = g_strdup (icon_name);
-	p = strrchr (icon_no_extension, '.');
-	if (p &&
-	    (strcmp (p, ".png") == 0 ||
-	     strcmp (p, ".xpm") == 0 ||
-	     strcmp (p, ".svg") == 0)) {
-	    *p = 0;
-	}
-
-	icon_info = gtk_icon_theme_lookup_icon (icon_theme,
-						icon_no_extension,
-						size,
-						0);
-	if (icon_info != NULL) {
-		retval = g_strdup (gtk_icon_info_get_filename (icon_info));
-		gtk_icon_info_free (icon_info);
-	}
-	
-	g_free (icon_no_extension);
-
-	return retval;
-}
-
-
-GdkPixbuf *
-create_pixbuf (GtkIconTheme  *icon_theme,
-	       const char    *icon_name,
-	       int            icon_size)
-{
-	char      *icon_path;
-	GdkPixbuf *pixbuf;
-	int        iw;
-	int        ih;
-
-	icon_path = panel_find_icon (icon_theme, icon_name, icon_size);
-	if (icon_path == NULL)
-		return NULL;
-
-	pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
-	g_free (icon_path);
-
-	if (pixbuf == NULL)
-		return NULL;
-
-	iw = gdk_pixbuf_get_width (pixbuf);
-	ih = gdk_pixbuf_get_height (pixbuf);
-
-	if ((iw > icon_size) || (ih > icon_size)) {
-		GdkPixbuf *scaled;
-		gdouble    factor;
-		gdouble    max = icon_size;
-		int        new_w, new_h;
-
-		factor = MIN (max / iw, max / ih);
-		new_w  = MAX ((int) (iw * factor), 1);
-		new_h = MAX ((int) (ih * factor), 1);
-
-		scaled = gdk_pixbuf_scale_simple (pixbuf,
-						  new_w,
-						  new_h,
-						  GDK_INTERP_BILINEAR);
-		g_object_unref (pixbuf);
-		pixbuf = scaled;
-	}
-
-	return pixbuf;
-}
-
-
-GdkPixbuf *
-get_icon_pixbuf (GtkIconTheme *icon_theme,
-		 GThemedIcon  *icon,
-		 int           size) 
+static GdkPixbuf *
+get_themed_icon_pixbuf (GThemedIcon  *icon,
+		        int           size,
+		        GtkIconTheme *icon_theme)
 {
 	char        **icon_names;
 	GtkIconInfo  *icon_info;
 	GdkPixbuf    *pixbuf;
 	GError       *error = NULL;
-	
+
 	g_object_get (icon, "names", &icon_names, NULL);
-	
+
 	icon_info = gtk_icon_theme_choose_icon (icon_theme, (const char **)icon_names, size, 0);
-	if (icon_info == NULL) {
-		GIcon *default_icon;
-		
-		default_icon = g_content_type_get_icon ("gnome-fs-regular");
-		pixbuf = get_icon_pixbuf (icon_theme, G_THEMED_ICON (default_icon), size);
-		g_object_unref (default_icon);
-		
-		return pixbuf;
-	}
-		
+	if (icon_info == NULL)
+		icon_info = gtk_icon_theme_lookup_icon (icon_theme, "text-x-generic", size, GTK_ICON_LOOKUP_USE_BUILTIN);
+
 	pixbuf = gtk_icon_info_load_icon (icon_info, &error);
 	if (pixbuf == NULL) {
 		g_warning ("could not load icon pixbuf: %s\n", error->message);
 		g_clear_error (&error);
 	}
-	
+
 	gtk_icon_info_free (icon_info);
 	g_strfreev (icon_names);
+
+	return pixbuf;
+}
+
+
+static GdkPixbuf *
+get_file_icon_pixbuf (GFileIcon *icon,
+		      int        size)
+{
+	GFile     *file;
+	char      *filename;
+	GdkPixbuf *pixbuf;
+	
+	file = g_file_icon_get_file (icon);
+	filename = g_file_get_path (file);
+	pixbuf = gdk_pixbuf_new_from_file_at_size (filename, size, -1, NULL);
+	g_free (filename);
+	g_object_unref (file);
 	
 	return pixbuf;
 }
 
 
+GdkPixbuf *
+get_icon_pixbuf (GIcon        *icon,
+		 int           size,
+		 GtkIconTheme *theme)
+{
+	if (icon == NULL)
+		return NULL;
+	if (G_IS_THEMED_ICON (icon))
+		return get_themed_icon_pixbuf (G_THEMED_ICON (icon), size, theme);
+	if (G_IS_FILE_ICON (icon))
+		return get_file_icon_pixbuf (G_FILE_ICON (icon), size);
+	return NULL;
+}
+
+
 GdkPixbuf *
 get_mime_type_pixbuf (const char   *mime_type,
 		      int           icon_size,
@@ -836,14 +720,14 @@ get_mime_type_pixbuf (const char   *mime_type,
 {
 	GdkPixbuf *pixbuf = NULL;
 	GIcon     *icon;
-	
-	if (icon_theme == NULL) 
+
+	if (icon_theme == NULL)
 		icon_theme = gtk_icon_theme_get_default ();
-	
+
 	icon = g_content_type_get_icon (mime_type);
-	pixbuf = get_icon_pixbuf (icon_theme, G_THEMED_ICON (icon), icon_size);
+	pixbuf = get_icon_pixbuf (icon, icon_size, icon_theme);
 	g_object_unref (icon);
-	
+
 	return pixbuf;
 }
 
@@ -860,22 +744,42 @@ get_folder_pixbuf_size_for_list (GtkWidget *widget)
 }
 
 
+gboolean
+show_uri (GdkScreen   *screen,
+	  const char  *uri,
+	  guint32      timestamp,
+	  GError     **error)
+{
+	gboolean result;
+
+#ifndef GTK_2_13
+	GAppLaunchContext *app_context;
+
+	app_context = g_app_launch_context_new ();
+	result = g_app_info_launch_default_for_uri (uri, G_APP_LAUNCH_CONTEXT (app_context), error);
+
+	g_object_unref (app_context);
+#else
+	result = gtk_show_uri (screen, uri, timestamp, error);
+#endif
+
+	return result;
+}
+
+
 void
 show_help_dialog (GtkWindow  *parent,
 		  const char *section)
 {
-	
-	GdkAppLaunchContext *app_context;
-	char                *uri;
-	GError              *error = NULL;
-	
-	uri = g_strconcat ("ghelp:file-roller", section ? "?" : NULL, section, NULL);	
-	app_context = gdk_app_launch_context_new ();
-	if (! g_app_info_launch_default_for_uri (uri, G_APP_LAUNCH_CONTEXT (app_context), &error)) {
+	char   *uri;
+	GError *error = NULL;
+
+	uri = g_strconcat ("ghelp:file-roller", section ? "?" : NULL, section, NULL);
+	if (! show_uri (gtk_window_get_screen (parent), uri, GDK_CURRENT_TIME, &error)) {
   		GtkWidget *dialog;
 
 		dialog = _gtk_message_dialog_new (parent,
-						  GTK_DIALOG_DESTROY_WITH_PARENT, 
+						  GTK_DIALOG_DESTROY_WITH_PARENT,
 						  GTK_STOCK_DIALOG_ERROR,
 						  _("Could not display help"),
 						  error->message,
@@ -893,6 +797,5 @@ show_help_dialog (GtkWindow  *parent,
 
 		g_clear_error (&error);
 	}
-	g_object_unref (app_context);
 	g_free (uri);
 }
